@@ -1,7 +1,7 @@
 /*
 ** ypserv.c - YP server main routines.
 **
-** Copyright (c) 1996, 1997, 1998 Thorsten Kukuk
+** Copyright (c) 1996, 1997, 1998, 1999, 2000 Thorsten Kukuk
 ** Copyright (c) 1993 Signum Support AB, Sweden
 **
 ** This file is part of the NYS YP Server.
@@ -22,13 +22,9 @@
 ** Cambridge, MA 02139, USA.
 **
 ** Author: Peter Eriksson <pen@signum.se>
-** Modified: Thorsten Kukuk <kukuk@uni-paderborn.de>
+** Modified: Thorsten Kukuk <kukuk@suse.de>
 */
 
-#ifndef LINT
-static const char rcsid[] = "$Id: ypserv.c,v 1.14 1998/03/08 16:05:40 kukuk Exp $";
-#endif
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -352,47 +348,45 @@ ypprog_2(struct svc_req *rqstp, register SVCXPRT *transp)
   return;
 }
 
-void ypserv_svc_run(void)
+extern int forked;
+
+void
+ypserv_svc_run (void)
 {
 #ifdef FD_SETSIZE
   fd_set readfds;
 #else
   int readfds;
 #endif /* def FD_SETSIZE */
-  extern int forked;
-  int pid;
 
-  /* Establish the identity of the parent ypserv process. */
-  pid = getpid();
-
-  for (;;) {
+  for (;;)
+    {
 #ifdef FD_SETSIZE
-    readfds = svc_fdset;
+      readfds = svc_fdset;
 #else
-    readfds = svc_fds;
+      readfds = svc_fds;
 #endif /* def FD_SETSIZE */
-    switch (select(_rpc_dtablesize(), &readfds, NULL, NULL,
-		   (struct timeval *)0))
-      {
-      case -1:
-	if (errno == EINTR)
+      switch (select(_rpc_dtablesize(), &readfds, NULL, NULL,
+		     (struct timeval *)0))
+	{
+	case -1:
+	  if (errno == EINTR)
 	    continue;
-	syslog(LOG_ERR, "svc_run: - select failed: %m");
-	return;
-      case 0:
-	continue;
-      default:
-	svc_getreqset(&readfds);
-	if (forked && pid != getpid())
-	  exit(0);
-      }
-  }
+	  syslog (LOG_ERR, "svc_run: - select failed: %m");
+	  return;
+	case 0:
+	  continue;
+	default:
+	  svc_getreqset (&readfds);
+	  if (forked)
+	    exit (0);
+	}
+    }
 }
 
-/*
-** Create a pidfile on startup
-*/
-void create_pidfile(void)
+/* Create a pidfile on startup */
+static void
+create_pidfile(void)
 {
   int fd, left, written, flags;
   pid_t pid;
@@ -467,50 +461,60 @@ void create_pidfile(void)
   return;
 }
 
-/*
-** Clean up after child processes signal their termination.
-*/
-void sig_child(int sig)
+/* Clean up after child processes signal their termination. */
+static void
+sig_child (int sig)
 {
-   int st;
+  int st;
+  pid_t pid;
 
-#ifdef HAVE_WAIT3
-   while (wait3(&st, WNOHANG, NULL) > 0)
-#else
-   while (waitpid(-1, &st, WNOHANG) > 0)
-#endif
-      children--;
+  if (debug_flag)
+    yp_msg ("sig_child: got signal %i\n", sig);
+
+  while ((pid = waitpid (-1, &st, WNOHANG)) > 0)
+    {
+      yp_msg ("pid=%d\n", pid);
+      --children;
+    }
+
+  if (children < 0)
+    {
+      yp_msg ("children is lower 0 (%i)!\n", children);
+      abort ();
+      children = 0;
+    }
+  else if (debug_flag)
+    yp_msg ("children = %i\n", children);
 }
 
-/*
-** Clean up if we quit the program.
-*/
-void sig_quit(int sig)
+/* Clean up if we quit the program. */
+static void
+sig_quit (int sig __attribute__ ((unused)))
 {
-  pmap_unset(YPPROG, YPVERS);
-  pmap_unset(YPPROG, YPOLDVERS);
-  unlink(_YPSERV_PIDFILE);
+  pmap_unset (YPPROG, YPVERS);
+  pmap_unset (YPPROG, YPOLDVERS);
+  unlink (_YPSERV_PIDFILE);
 
-  exit(0);
+  exit (0);
 }
 
-/*
-** Reload securenets and config file
-*/
-void sig_hup(int sig)
+/* Reload securenets and config file */
+static void
+sig_hup (int sig __attribute__ ((unused)))
 {
 #ifndef HAVE_LIBWRAP
-  load_securenets();
+  load_securenets ();
 #endif
-  load_config();
+  load_config ();
 }
 
-void Usage(int exitcode)
+static void
+Usage (int exitcode)
 {
   fputs ("usage: ypserv [-d [path]] [-b] [-p port]\n", stderr);
   fputs ("       ypserv --version\n", stderr);
 
-  exit(exitcode);
+  exit (exitcode);
 }
 
 int
@@ -551,7 +555,7 @@ main (int argc, char **argv)
         case 'v':
 	  debug_flag=1;
 	  yp_msg("ypserv - NYS YP Server version %s (with %s)\n", version,
-#if HAVE_LIBWRAP
+#ifdef HAVE_LIBWRAP
 		 "tcp wrapper"
 #else
 		 "securenets"
@@ -585,7 +589,7 @@ main (int argc, char **argv)
   if (debug_flag)
     yp_msg("[Welcome to the NYS YP Server, version %s (with %s)]\n",
 	   version,
-#if HAVE_LIBWRAP
+#ifdef HAVE_LIBWRAP
 	   "tcp wrapper"
 #else
 	   "securenets"
@@ -595,14 +599,38 @@ main (int argc, char **argv)
     {
       int i;
 
-      if (fork())
+      if ((i = fork()) > 0)
 	exit(0);
 
-      for (i = 0; i < 255; i++)
+      if (i < 0)
+	{
+	  yp_msg ("Cannot fork: %s\n", strerror (errno));
+	  exit (-1);
+	}
+
+      if (setsid() == -1)
+	{
+	  yp_msg ("Cannot setsid: %s\n", strerror (errno));
+	  exit (-1);
+	}
+
+      if ((i = fork()) > 0)
+	exit(0);
+
+      if (i < 0)
+	{
+	  yp_msg ("Cannot fork: %s\n", strerror (errno));
+	  exit (-1);
+	}
+
+      for (i = 0; i < getdtablesize(); ++i)
 	close(i);
+      errno = 0;
 
-      if (fork())
-	exit(0);
+      umask(0);
+      i = open("/dev/null", O_RDWR);
+      dup(i);
+      dup(i);
     }
 
   if (argc > 0)
