@@ -1,10 +1,6 @@
 #ifndef __YPSERV_H__
 #define __YPSERV_H__
 
-/*
-** $Id: ypserv.h,v 1.6 1998/02/04 20:49:19 kukuk Exp $
-*/
-
 #include <rpc/rpc.h>
 
 /* Struct for ypserv.conf options */
@@ -23,7 +19,7 @@ extern char *progname;      	/* ypserv.c */
 extern char *path_ypdb;		/* ypserv.c */
 extern int debug_flag;		/* ypserv.c || ypxfrd.c */
 extern int dns_flag;		/* ypserv.c || ypxfrd.c */
-extern int children;		/* server.c || ypxfrd.c */
+extern volatile int children;		/* server.c || ypxfrd.c */
 extern int forked;		/* server.c || ypxfrd.c */
 extern int xfr_check_port;	/* server.c || ypxfrd.c */
 
